@@ -6,6 +6,8 @@
   Copyright information is at the end of the file
 =============================================================================*/
 
+#define _XOPEN_SOURCE 600  /* Make sure strdup() is in <string.h> */
+
 #include <ctype.h>
 #include <assert.h>
 #include <stdlib.h>
@@ -15,12 +17,17 @@
 #include <time.h>
 
 #include "xmlrpc_config.h"
+#include "bool.h"
+#include "int.h"
+#include "version.h"
 #include "mallocvar.h"
 #include "xmlrpc-c/string_int.h"
 #include "xmlrpc-c/abyss.h"
 
+#include "trace.h"
 #include "server.h"
 #include "session.h"
+#include "file.h"
 #include "conn.h"
 #include "token.h"
 #include "date.h"
@@ -31,9 +38,9 @@
 
 
 void
-ResponseError(TSession * const sessionP) {
+ResponseError2(TSession *   const sessionP,
+               const char * const explanation) {
 
-    const char * const reason = HTTPReasonByStatus(sessionP->status);
     const char * errorDocument;
 
     ResponseAddField(sessionP, "Content-type", "text/html");
@@ -42,47 +49,66 @@ ResponseError(TSession * const sessionP) {
     
     xmlrpc_asprintf(&errorDocument,
                     "<HTML><HEAD><TITLE>Error %d</TITLE></HEAD>"
-                    "<BODY><H1>Error %d</H1><P>%s</P>" SERVER_HTML_INFO 
-                    "</BODY></HTML>",
-                    sessionP->status, sessionP->status, reason);
+                    "<BODY>"
+                    "<H1>Error %d</H1>"
+                    "<P>%s</P>" SERVER_HTML_INFO 
+                    "</BODY>"
+                    "</HTML>",
+                    sessionP->status, sessionP->status, explanation);
     
-    ConnWrite(sessionP->conn, errorDocument, strlen(errorDocument)); 
+    ConnWrite(sessionP->connP, errorDocument, strlen(errorDocument)); 
 
     xmlrpc_strfree(errorDocument);
 }
 
 
 
+void
+ResponseError(TSession * const sessionP) {
+
+    ResponseError2(sessionP, HTTPReasonByStatus(sessionP->status));
+}
+
+
+
+
 abyss_bool
 ResponseChunked(TSession * const sessionP) {
-    /* This is only a hope, things will be real only after a call of
-       ResponseWriteStart()
-    */
+/*----------------------------------------------------------------------------
+   Make the response chunked.
+
+   This is only a hope, things will be real only after a call of
+   ResponseWriteStart()
+-----------------------------------------------------------------------------*/
     assert(!sessionP->responseStarted);
 
     sessionP->chunkedwrite =
         (sessionP->version.major > 1) ||
         (sessionP->version.major == 1 && (sessionP->version.minor >= 1));
 
-    sessionP->chunkedwritemode = TRUE;
+    sessionP->chunkedwritemode = true;
 
-    return TRUE;
+    return true;
 }
 
 
 
 void
-ResponseStatus(TSession * const sessionP,
-               uint16_t   const code) {
-
+ResponseStatus(TSession *     const sessionP,
+               unsigned short const code) {
+/*----------------------------------------------------------------------------
+   Set the status code for the response to 'code'.
+-----------------------------------------------------------------------------*/
     sessionP->status = code;
 }
 
 
 
-uint16_t
+xmlrpc_uint16_t
 ResponseStatusFromErrno(int const errnoArg) {
-
+/*----------------------------------------------------------------------------
+   The appropriate HTTP status code for a POSIX errno of 'errnoArg'.
+-----------------------------------------------------------------------------*/
     uint16_t code;
 
     switch (errnoArg) {
@@ -102,59 +128,72 @@ ResponseStatusFromErrno(int const errnoArg) {
 
 void
 ResponseStatusErrno(TSession * const sessionP) {
-
+/*----------------------------------------------------------------------------
+   Set the response state from the global 'errno' value (i.e. the result
+   of the last system call Caller made).
+-----------------------------------------------------------------------------*/
     ResponseStatus(sessionP, ResponseStatusFromErrno(errno));
 }
 
 
 
+static bool
+isValidHttpToken(const char * const token) {
+
+    char const separators[] = "()<>@,;:\\\"/[]?={} \t";
+    const char * p;
+    bool valid;
+
+    for (p = &token[0], valid = true; *p; ++p) {
+        if (!isprint(*p) || strchr(separators, *p))
+            valid = false;
+    }
+    return valid;
+}
+
+
+
+
+static bool
+isValidHttpText(const char * const text) {
+
+    const char * p;
+    bool valid;
+
+    for (p = &text[0], valid = true; *p; ++p) {
+        if (!isprint(*p))
+            valid = false;
+    }
+    return valid;
+}
+
+
+
 abyss_bool
 ResponseAddField(TSession *   const sessionP,
                  const char * const name,
                  const char * const value) {
 
-    return TableAdd(&sessionP->response_headers, name, value);
+    abyss_bool succeeded;
+    
+    if (!isValidHttpToken(name)) {
+        TraceMsg("Supplied HTTP header field name is not a valid HTTP token");
+        succeeded = false;
+    } else if (!isValidHttpText(value)) {
+        TraceMsg("Supplied HTTP header field value is not valid HTTP text");
+        succeeded = false;
+    } else {
+        succeeded = TableAdd(&sessionP->responseHeaderFields, name, value);
+    }
+    return succeeded;
 }
 
 
 
 static void
-addDateHeader(TSession * const sessionP) {
+addConnectionHeaderFld(TSession * const sessionP) {
 
-    char dateValue[64];
-    abyss_bool validDate;
-
-    validDate = DateToString(&sessionP->date, dateValue);
-
-    if (sessionP->status >= 200 && validDate)
-        ResponseAddField(sessionP, "Date", dateValue);
-}
-
-
-
-void
-ResponseWriteStart(TSession * const sessionP) {
-
-    struct _TServer * const srvP = ConnServer(sessionP->conn)->srvP;
-
-    unsigned int i;
-
-    assert(!sessionP->responseStarted);
-
-    if (sessionP->status == 0) {
-        // Handler hasn't set status.  That's an error
-        sessionP->status = 500;
-    }
-
-    sessionP->responseStarted = TRUE;
-
-    {
-        const char * const reason = HTTPReasonByStatus(sessionP->status);
-        const char * line;
-        xmlrpc_asprintf(&line,"HTTP/1.1 %u %s\r\n", sessionP->status, reason);
-        ConnWrite(sessionP->conn, line, strlen(line));
-        xmlrpc_strfree(line);
-    }
+    struct _TServer * const srvP = ConnServer(sessionP->connP)->srvP;
 
     if (HTTPKeepalive(sessionP)) {
         const char * keepaliveValue;
@@ -169,34 +208,178 @@ ResponseWriteStart(TSession * const sessionP) {
         xmlrpc_strfree(keepaliveValue);
     } else
         ResponseAddField(sessionP, "Connection", "close");
+}
     
-    if (sessionP->chunkedwrite && sessionP->chunkedwritemode)
-        ResponseAddField(sessionP, "Transfer-Encoding", "chunked");
 
-    addDateHeader(sessionP);
 
-    /* Generation of the server field */
-    if (srvP->advertise)
-        ResponseAddField(sessionP, "Server", SERVER_HVERSION);
+static void
+addDateHeaderFld(TSession * const sessionP) {
+
+    if (sessionP->status >= 200) {
+        const char * dateValue;
+
+        DateToString(sessionP->date, &dateValue);
+
+        if (dateValue) {
+            ResponseAddField(sessionP, "Date", dateValue);
+            xmlrpc_strfree(dateValue);
+        }
+    }
+}
+
+
+
+static void
+addServerHeaderFld(TSession * const sessionP) {
+
+    const char * serverValue;
+
+    xmlrpc_asprintf(&serverValue, "Xmlrpc-c_Abyss/%s", XMLRPC_C_VERSION);
+
+    ResponseAddField(sessionP, "Server", serverValue);
+
+    xmlrpc_strfree(serverValue);
+}
+
+
+
+static unsigned int
+leadingWsCt(const char * const arg) {
+
+    unsigned int i;
+
+    for (i = 0; arg[i] && isspace(arg[i]); ++i);
+
+    return i;
+}
+
+
+
+static unsigned int
+trailingWsPos(const char * const arg) {
+
+    unsigned int i;
+
+    for (i = strlen(arg); i > 0 && isspace(arg[i-1]); --i);
+
+    return i;
+}
+
+
+
+static const char *
+formatFieldValue(const char * const unformatted) {
+/*----------------------------------------------------------------------------
+   Return the string of characters that goes after the colon on the
+   HTTP header field line, given that 'unformatted' is its basic value.
+-----------------------------------------------------------------------------*/
+    const char * retval;
+
+    /* An HTTP header field value may not have leading or trailing white
+       space.
+    */
+    char * buffer;
+
+    buffer = malloc(strlen(unformatted) + 1);
+
+    if (buffer == NULL)
+        retval = xmlrpc_strnomemval();
+    else {
+        unsigned int const lead  = leadingWsCt(unformatted);
+        unsigned int const trail = trailingWsPos(unformatted);
+        assert(trail >= lead);
+        strncpy(buffer, &unformatted[lead], trail - lead);
+        buffer[trail - lead] = '\0';
+        retval = buffer;
+    }
+    return retval;
+}
+
+
+
+static void
+sendHeader(TConn * const connP,
+           TTable  const fields) {
+/*----------------------------------------------------------------------------
+   Send the HTTP response header whose fields are fields[].
+
+   Don't include the blank line that separates the header from the body.
+
+   fields[] contains syntactically valid HTTP header field names and values.
+   But to the extent that int contains undefined field names or semantically
+   invalid values, the header we send is invalid.
+-----------------------------------------------------------------------------*/
+    unsigned int i;
+
+    for (i = 0; i < fields.size; ++i) {
+        TTableItem * const fieldP = &fields.item[i];
+        const char * const fieldValue = formatFieldValue(fieldP->value);
 
-    /* send all the fields */
-    for (i = 0; i < sessionP->response_headers.size; ++i) {
-        TTableItem * const ti = &sessionP->response_headers.item[i];
         const char * line;
-        xmlrpc_asprintf(&line, "%s: %s\r\n", ti->name, ti->value);
-        ConnWrite(sessionP->conn, line, strlen(line));
+
+        xmlrpc_asprintf(&line, "%s: %s\r\n", fieldP->name, fieldValue);
+        ConnWrite(connP, line, strlen(line));
+        xmlrpc_strfree(line);
+        xmlrpc_strfree(fieldValue);
+    }
+}
+
+
+void
+ResponseWriteStart(TSession * const sessionP) {
+/*----------------------------------------------------------------------------
+   Begin the process of sending the response for an HTTP transaction
+   (i.e. Abyss session).
+
+   As part of this, send the entire HTTP header for the response.
+-----------------------------------------------------------------------------*/
+    struct _TServer * const srvP = ConnServer(sessionP->connP)->srvP;
+
+    assert(!sessionP->responseStarted);
+
+    if (sessionP->status == 0) {
+        /* Handler hasn't set status.  That's an error */
+        TraceMsg("Abyss client called ResponseWriteStart() on "
+                 "a session for which he has not set the request status "
+                 "('status' member of TSession).  Using status 500\n");
+        sessionP->status = 500;
+    }
+
+    sessionP->responseStarted = true;
+
+    {
+        const char * const reason = HTTPReasonByStatus(sessionP->status);
+        const char * line;
+        xmlrpc_asprintf(&line,"HTTP/1.1 %u %s\r\n", sessionP->status, reason);
+        ConnWrite(sessionP->connP, line, strlen(line));
         xmlrpc_strfree(line);
     }
 
-    ConnWrite(sessionP->conn, "\r\n", 2);  
+    addConnectionHeaderFld(sessionP);
+
+    if (sessionP->chunkedwrite && sessionP->chunkedwritemode)
+        ResponseAddField(sessionP, "Transfer-Encoding", "chunked");
+
+    addDateHeaderFld(sessionP);
+
+    if (srvP->advertise)
+        addServerHeaderFld(sessionP);
+
+    /* Note that sessionP->responseHeaderFields is defined to contain
+       syntactically but not necessarily semantically valid header
+       field names and values.
+    */
+    sendHeader(sessionP->connP, sessionP->responseHeaderFields);
+
+    ConnWrite(sessionP->connP, "\r\n", 2);  
 }
 
 
 
 abyss_bool
-ResponseWriteBody(TSession *   const sessionP,
-                  const char * const data,
-                  uint32_t     const len) {
+ResponseWriteBody(TSession *      const sessionP,
+                  const char *    const data,
+                  xmlrpc_uint32_t const len) {
 
     return HTTPWriteBodyChunk(sessionP, data, len);
 }
@@ -221,16 +404,41 @@ ResponseContentType(TSession *   const serverP,
 
 
 abyss_bool
-ResponseContentLength(TSession * const sessionP,
-                      uint64_t   const len) {
+ResponseContentLength(TSession *      const sessionP,
+                      xmlrpc_uint64_t const len) {
+
     char contentLengthValue[32];
     
-    sprintf(contentLengthValue, "%llu", len);
+    sprintf(contentLengthValue, "%" PRIu64, len);
 
     return ResponseAddField(sessionP, "Content-length", contentLengthValue);
 }
 
 
+
+void
+ResponseAccessControl(TSession *        const abyssSessionP, 
+                      ResponseAccessCtl const accessControl) {
+
+    if (accessControl.allowOrigin) {
+        ResponseAddField(abyssSessionP, "Access-Control-Allow-Origin",
+                         accessControl.allowOrigin);
+        ResponseAddField(abyssSessionP, "Access-Control-Allow-Methods",
+                         "POST");
+        ResponseAddField(abyssSessionP, "Access-Control-Allow-Headers",
+                         "Content-Type");
+        ResponseAddField(abyssSessionP, "Access-Control-Allow-Headers",
+                         "Content-Length");
+        if (accessControl.expires) {
+            char buffer[64];
+            sprintf(buffer, "%u", accessControl.maxAge);
+            ResponseAddField(abyssSessionP, "Access-Control-Max-Age", buffer);
+        }
+    }
+}
+
+
+
 /*********************************************************************
 ** MIMEType
 *********************************************************************/
@@ -301,11 +509,11 @@ static void
 mimeTypeAdd(MIMEType *   const MIMETypeP,
             const char * const type,
             const char * const ext,
-            abyss_bool * const successP) {
+            bool *       const successP) {
     
     uint16_t index;
     void * mimeTypesItem;
-    abyss_bool typeIsInList;
+    bool typeIsInList;
 
     assert(MIMETypeP != NULL);
 
@@ -316,34 +524,34 @@ mimeTypeAdd(MIMEType *   const MIMETypeP,
         mimeTypesItem = (void*)PoolStrdup(&MIMETypeP->pool, type);
 
     if (mimeTypesItem) {
-        abyss_bool extIsInList;
+        bool extIsInList;
         extIsInList = ListFindString(&MIMETypeP->extList, ext, &index);
         if (extIsInList) {
             MIMETypeP->typeList.item[index] = mimeTypesItem;
-            *successP = TRUE;
+            *successP = true;
         } else {
             void * extItem = (void*)PoolStrdup(&MIMETypeP->pool, ext);
             if (extItem) {
-                abyss_bool addedToMimeTypes;
+                bool addedToMimeTypes;
 
                 addedToMimeTypes =
                     ListAdd(&MIMETypeP->typeList, mimeTypesItem);
                 if (addedToMimeTypes) {
-                    abyss_bool addedToExt;
+                    bool addedToExt;
                     
                     addedToExt = ListAdd(&MIMETypeP->extList, extItem);
                     *successP = addedToExt;
                     if (!*successP)
                         ListRemove(&MIMETypeP->typeList);
                 } else
-                    *successP = FALSE;
+                    *successP = false;
                 if (!*successP)
                     PoolReturn(&MIMETypeP->pool, extItem);
             } else
-                *successP = FALSE;
+                *successP = false;
         }
     } else
-        *successP = FALSE;
+        *successP = false;
 }
 
 
@@ -356,10 +564,10 @@ MIMETypeAdd2(MIMEType *   const MIMETypeArg,
 
     MIMEType * MIMETypeP = MIMETypeArg ? MIMETypeArg : globalMimeTypeP;
 
-    abyss_bool success;
+    bool success;
 
     if (MIMETypeP == NULL)
-        success = FALSE;
+        success = false;
     else 
         mimeTypeAdd(MIMETypeP, type, ext, &success);
 
@@ -383,7 +591,7 @@ mimeTypeFromExt(MIMEType *   const MIMETypeP,
 
     const char * retval;
     uint16_t extindex;
-    abyss_bool extIsInList;
+    bool extIsInList;
 
     assert(MIMETypeP != NULL);
 
@@ -430,19 +638,19 @@ findExtension(const char *  const fileName,
 
     unsigned int extPos = 0;  /* stifle unset variable warning */
         /* Running estimation of where in fileName[] the extension starts */
-    abyss_bool extFound;
+    bool extFound;
     unsigned int i;
 
     /* We're looking for the last dot after the last slash */
-    for (i = 0, extFound = FALSE; fileName[i]; ++i) {
+    for (i = 0, extFound = false; fileName[i]; ++i) {
         char const c = fileName[i];
         
         if (c == '.') {
-            extFound = TRUE;
+            extFound = true;
             extPos = i + 1;
         }
         if (c == '/')
-            extFound = FALSE;
+            extFound = false;
     }
 
     if (extFound)
@@ -500,42 +708,42 @@ MIMETypeFromFileName(const char * const fileName) {
 
 
 
-static abyss_bool
+static bool
 fileContainsText(const char * const fileName) {
 /*----------------------------------------------------------------------------
    Return true iff we can read the contents of the file named 'fileName'
    and see that it appears to be composed of plain text characters.
 -----------------------------------------------------------------------------*/
-    abyss_bool retval;
-    abyss_bool fileOpened;
-    TFile file;
+    bool retval;
+    bool fileOpened;
+    TFile * fileP;
 
-    fileOpened = FileOpen(&file, fileName, O_BINARY | O_RDONLY);
+    fileOpened = FileOpen(&fileP, fileName, O_BINARY | O_RDONLY);
     if (fileOpened) {
         char const ctlZ = 26;
         unsigned char buffer[80];
         int32_t readRc;
         unsigned int i;
 
-        readRc = FileRead(&file, buffer, sizeof(buffer));
+        readRc = FileRead(fileP, buffer, sizeof(buffer));
        
         if (readRc >= 0) {
             unsigned int bytesRead = readRc;
-            abyss_bool nonTextFound;
+            bool nonTextFound;
 
-            nonTextFound = FALSE;  /* initial value */
+            nonTextFound = false;  /* initial value */
     
             for (i = 0; i < bytesRead; ++i) {
                 char const c = buffer[i];
                 if (c < ' ' && !isspace(c) && c != ctlZ)
-                    nonTextFound = TRUE;
+                    nonTextFound = true;
             }
             retval = !nonTextFound;
         } else
-            retval = FALSE;
-        FileClose(&file);
+            retval = false;
+        FileClose(fileP);
     } else
-        retval = FALSE;
+        retval = false;
 
     return retval;
 }
@@ -585,47 +793,6 @@ MIMETypeGuessFromFile(const char * const fileName) {
 
                                   
 
-/*********************************************************************
-** Base64
-*********************************************************************/
-
-void Base64Encode(char *s,char *d)
-{
-    /* Conversion table. */
-    static char tbl[64] = {
-        'A','B','C','D','E','F','G','H',
-        'I','J','K','L','M','N','O','P',
-        'Q','R','S','T','U','V','W','X',
-        'Y','Z','a','b','c','d','e','f',
-        'g','h','i','j','k','l','m','n',
-        'o','p','q','r','s','t','u','v',
-        'w','x','y','z','0','1','2','3',
-        '4','5','6','7','8','9','+','/'
-    };
-
-    uint32_t i,length=strlen(s);
-    char *p=d;
-    
-    /* Transform the 3x8 bits to 4x6 bits, as required by base64. */
-    for (i = 0; i < length; i += 3)
-    {
-        *p++ = tbl[s[0] >> 2];
-        *p++ = tbl[((s[0] & 3) << 4) + (s[1] >> 4)];
-        *p++ = tbl[((s[1] & 0xf) << 2) + (s[2] >> 6)];
-        *p++ = tbl[s[2] & 0x3f];
-        s += 3;
-    }
-    
-    /* Pad the result if necessary... */
-    if (i == length + 1)
-        *(p - 1) = '=';
-    else if (i == length + 2)
-        *(p - 1) = *(p - 2) = '=';
-    
-    /* ...and zero-terminate it. */
-    *p = '\0';
-}
-
 /******************************************************************************
 **
 ** http.c
