@@ -7,6 +7,7 @@
 
 #include "xmlrpc_config.h"
 
+#include <assert.h>
 #include <stddef.h>
 #include <stdlib.h>
 
@@ -24,21 +25,21 @@ xmlrpc_abort_if_array_bad(xmlrpc_value * const arrayP) {
     else if (arrayP->_type != XMLRPC_TYPE_ARRAY)
         abort();
     else {
-        unsigned int const arraySize =
-            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value*, &arrayP->_block);
+        size_t const arraySize =
+            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, arrayP->blockP);
         xmlrpc_value ** const contents = 
-            XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value*, &arrayP->_block);
+            XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, arrayP->blockP);
         
         if (contents == NULL)
             abort();
         else {
-            unsigned int index;
+            size_t index;
             
             for (index = 0; index < arraySize; ++index) {
                 xmlrpc_value * const itemP = contents[index];
                 if (itemP == NULL)
                     abort();
-                else if (itemP->_refcount < 1)
+                else if (itemP->refcount < 1)
                     abort();
             }
         }
@@ -53,12 +54,12 @@ xmlrpc_destroyArrayContents(xmlrpc_value * const arrayP) {
    Dispose of the contents of an array (but not the array value itself).
    The value is not valid after this.
 -----------------------------------------------------------------------------*/
-    unsigned int const arraySize =
-        XMLRPC_MEMBLOCK_SIZE(xmlrpc_value*, &arrayP->_block);
+    size_t const arraySize =
+        XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, arrayP->blockP);
     xmlrpc_value ** const contents = 
-        XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value*, &arrayP->_block);
+        XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, arrayP->blockP);
 
-    unsigned int index;
+    size_t index;
     
     XMLRPC_ASSERT_ARRAY_OK(arrayP);
 
@@ -67,31 +68,33 @@ xmlrpc_destroyArrayContents(xmlrpc_value * const arrayP) {
         xmlrpc_value * const itemP = contents[index];
         xmlrpc_DECREF(itemP);
     }
-    XMLRPC_MEMBLOCK_CLEAN(xmlrpc_value *, &arrayP->_block);
+    XMLRPC_MEMBLOCK_FREE(xmlrpc_value *, arrayP->blockP);
 }
 
 
 
 int 
-xmlrpc_array_size(xmlrpc_env *         const env, 
-                  const xmlrpc_value * const array) {
+xmlrpc_array_size(xmlrpc_env *         const envP,
+                  const xmlrpc_value * const arrayP) {
 
     int retval;
 
-    /* Suppress a compiler warning about uninitialized variables. */
-    retval = 0;
+    XMLRPC_ASSERT_ENV_OK(envP);
+    XMLRPC_ASSERT_VALUE_OK(arrayP);
 
-    XMLRPC_ASSERT_ENV_OK(env);
-    XMLRPC_ASSERT_VALUE_OK(array);
-    XMLRPC_TYPE_CHECK(env, array, XMLRPC_TYPE_ARRAY);
+    if (arrayP->_type != XMLRPC_TYPE_ARRAY) {
+        xmlrpc_env_set_fault_formatted(
+            envP, XMLRPC_TYPE_ERROR, "Value is not an array");
+        retval = -1;
+    } else {
+        size_t const size =
+            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, arrayP->blockP);
 
-    retval = XMLRPC_TYPED_MEM_BLOCK_SIZE(xmlrpc_value*, &array->_block);
+        assert((size_t)(int)(size) == size);
 
-                  cleanup:
-    if (env->fault_occurred)
-        return -1;
-    else
-        return retval;
+        retval = (int)size;
+    }
+    return retval;
 }
 
 
@@ -109,13 +112,13 @@ xmlrpc_array_append_item(xmlrpc_env *   const envP,
             envP, XMLRPC_TYPE_ERROR, "Value is not an array");
     else {
         size_t const size = 
-            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, &arrayP->_block);
+            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, arrayP->blockP);
 
-        XMLRPC_MEMBLOCK_RESIZE(xmlrpc_value *, envP, &arrayP->_block, size+1);
+        XMLRPC_MEMBLOCK_RESIZE(xmlrpc_value *, envP, arrayP->blockP, size+1);
 
         if (!envP->fault_occurred) {
             xmlrpc_value ** const contents =
-                XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value*, &arrayP->_block);
+                XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, arrayP->blockP);
             xmlrpc_INCREF(valueP);
             contents[size] = valueP;
         }
@@ -140,9 +143,9 @@ xmlrpc_array_read_item(xmlrpc_env *         const envP,
             "a value that is not an array");
     else {
         xmlrpc_value ** const contents = 
-            XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, &arrayP->_block);
+            XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, arrayP->blockP);
         size_t const size = 
-            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, &arrayP->_block);
+            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, arrayP->blockP);
 
         if (index >= size)
             xmlrpc_env_set_fault_formatted(
@@ -160,50 +163,22 @@ xmlrpc_array_read_item(xmlrpc_env *         const envP,
 xmlrpc_value * 
 xmlrpc_array_get_item(xmlrpc_env *         const envP,
                       const xmlrpc_value * const arrayP,
-                      int                  const indexArg) {
-
-    /* We must maintain the historical thread-safeness of
-       xmlrpc_array_get_item().  That means we can't call
-       xmlrpc_read_array(), because it modifies the reference count
-       of its arguments, thus is not thread-safe.
-
-       The Xmlrpc-c method registry is an example of an application that
-       relies on thread-safeness of xmlrpc_array_get_item() -- it uses
-       xmlrpc_value's to represent the registry, and multiple server
-       threads read the registry simultaneously.
-    */
+                      int                  const index) {
 
     xmlrpc_value * valueP;
 
-    XMLRPC_ASSERT_ENV_OK(envP);
-    XMLRPC_ASSERT_VALUE_OK(arrayP);
-
-    valueP = NULL;
-
-    if (indexArg < 0)
+    if (index < 0)
         xmlrpc_env_set_fault_formatted(
-            envP, XMLRPC_INDEX_ERROR, "Index %d is negative.", indexArg);
+            envP, XMLRPC_INDEX_ERROR, "Index %d is negative.", index);
     else {
-        unsigned int const index = indexArg;
+        xmlrpc_array_read_item(envP, arrayP, index, &valueP);
 
-        if (arrayP->_type != XMLRPC_TYPE_ARRAY)
-            xmlrpc_env_set_fault_formatted(
-                envP, XMLRPC_TYPE_ERROR, "Attempt to read array item from "
-                "a value that is not an array");
-        else {
-            xmlrpc_value ** const contents = 
-                XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, &arrayP->_block);
-            size_t const size = 
-                XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, &arrayP->_block);
-
-            if (index >= size)
-                xmlrpc_env_set_fault_formatted(
-                    envP, XMLRPC_INDEX_ERROR, "Array index %u is beyond end "
-                    "of %u-item array", index, (unsigned int)size);
-            else
-                valueP = contents[index];
-        }
+        if (!envP->fault_occurred)
+            xmlrpc_DECREF(valueP);
     }
+    if (envP->fault_occurred)
+        valueP = NULL;
+        
     return valueP;
 }
 
@@ -219,7 +194,7 @@ xmlrpc_array_new(xmlrpc_env * const envP) {
     xmlrpc_createXmlrpcValue(envP, &arrayP);
     if (!envP->fault_occurred) {
         arrayP->_type = XMLRPC_TYPE_ARRAY;
-        XMLRPC_MEMBLOCK_INIT(xmlrpc_value*, envP, &arrayP->_block, 0);
+        arrayP->blockP = XMLRPC_MEMBLOCK_NEW(xmlrpc_value *, envP, 0);
         if (envP->fault_occurred)
             free(arrayP);
     }
@@ -228,6 +203,54 @@ xmlrpc_array_new(xmlrpc_env * const envP) {
 
 
 
+xmlrpc_value *
+xmlrpc_array_new_value(xmlrpc_env *   const envP,
+                       xmlrpc_value * const valueP) {
+
+    xmlrpc_value * arrayP;
+
+    if (valueP->_type != XMLRPC_TYPE_ARRAY) {
+        xmlrpc_env_set_fault_formatted(envP, XMLRPC_TYPE_ERROR,
+                                       "Value is not an array.  "
+                                       "It is type #%d", valueP->_type);
+        arrayP = NULL;
+    } else {
+        size_t const size = 
+            XMLRPC_MEMBLOCK_SIZE(xmlrpc_value *, valueP->blockP);
+
+        xmlrpc_createXmlrpcValue(envP, &arrayP);
+        if (!envP->fault_occurred) {
+            arrayP->_type = XMLRPC_TYPE_ARRAY;
+
+            arrayP->blockP = XMLRPC_MEMBLOCK_NEW(xmlrpc_value *, envP, 0);
+
+            if (envP->fault_occurred)
+                free(arrayP);
+            else {
+                xmlrpc_value ** const srcValuePList =
+                    XMLRPC_MEMBLOCK_CONTENTS(xmlrpc_value *, valueP->blockP);
+                
+                unsigned int i;
+            
+                for (i = 0; i < size && !envP->fault_occurred; ++i) {
+                    xmlrpc_value * const newEltP =
+                        xmlrpc_value_new(envP, srcValuePList[i]);
+                    if (!envP->fault_occurred)
+                        xmlrpc_array_append_item(envP, arrayP, newEltP);
+                }
+                if (envP->fault_occurred)
+                    xmlrpc_destroyArrayContents(arrayP);
+            }
+
+            if (envP->fault_occurred)
+                free(arrayP);
+        }
+    }
+    return arrayP;
+}
+
+
+
 /* Copyright (C) 2001 by First Peer, Inc. All rights reserved.
 ** Copyright (C) 2001 by Eric Kidd. All rights reserved.
 **
